----------------------------------------------------------------------------------
-- Company: UAM 
-- Design Name: display_ctrl
-- Project Name: stopwatch
-- Description: 
--   Provides 7-segment coded, multiplexed output for a 4-digit display
--   based on the 4 digit input data of the stopwatch.
--   Includes refresh anode cycling.  Includes a register to freeze the
--   display if UpdateDisplay input is '0'.
-- Dependencies: 
--   stopwatch_pkg
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

use work.stopwatch_pkg.all;   -- use project's package

-------------------------------------------------------------------------------
-- Entity

entity display_ctrl is
  generic (
           REFRESH_NCYCLES : integer := 100000 -- Cycles between anode changes
          );
  port (
        Clk           : in  std_logic;  -- Clock (rising edge)
        Reset         : in  std_logic;  -- Async reset, active high
        Hundredths    : in  std_logic_vector (3 downto 0); -- 1/100 s digit
        Tenths        : in  std_logic_vector (3 downto 0); -- 1/10 s digit
        Seconds       : in  std_logic_vector (3 downto 0); -- 1 s digit
        Tens          : in  std_logic_vector (2 downto 0); -- 10 s digit
        UpdateDisplay : in  std_logic;  -- If '0', keep display frozen
        Segments      : out std_logic_vector (7 downto 0); -- Segment,active 0
                                                           --  (dp, g downto a)
        Anodes        : out std_logic_vector (3 downto 0) -- Anodes, active 0
       );        
end display_ctrl;

-------------------------------------------------------------------------------
-- Architecture

architecture Behavioral of display_ctrl is

  constant COUNT_WIDTH : integer := log2_ceil (REFRESH_NCYCLES);
  
  --  Refreshing interval counter:
  signal counter : std_logic_vector (COUNT_WIDTH-1 downto 0);
  --  Anodes signals (independent signal to be able to read Anodes output):
  signal anodesInt : std_logic_vector (3 downto 0);
  --  Registered digit data to be kept when display is frozen:
  signal hundredthsReg : std_logic_vector (3 downto 0);
  signal tenthsReg     : std_logic_vector (3 downto 0);
  signal secondsReg    : std_logic_vector (3 downto 0);
  signal tensReg       : std_logic_vector (2 downto 0);
  --  Multiplexed digit data:
  signal digitData    : std_logic_vector (3 downto 0);

begin

  -- Cyclic refresh for the 4 digits of the display 
  process (Clk, Reset)
  begin
    if Reset = '1' then
      counter  <= (others => '0');
      anodesInt <= "1110";
    elsif rising_edge (Clk) then
      -- If interval reached, set next anode and clear counter,otherwise count:
      if counter = (REFRESH_NCYCLES-1) then
        counter  <= (others => '0');
        -- INSERT IN THE FOLLOWING LINE THE SENTENCE TO ROTATE anodesInt:
            -- rotate left
      else
        counter <= counter + 1;
      end if;
    end if;
  end process;
  
  Anodes <= anodesInt;

  -- Register counter digit data when UpdateDisplay is set,
  --   otherwise keep previous value:
  process (Clk, Reset)
  begin
    if Reset = '1' then
      hundredthsReg <= (others => '0');
      tenthsReg     <= (others => '0');
      secondsReg    <= (others => '0');
      tensReg       <= (others => '0');
    elsif Clk'event and Clk = '1' then
      if UpdateDisplay = '1' then
        hundredthsReg <= Hundredths;
        tenthsReg     <= Tenths;
        secondsReg    <= Seconds;
        tensReg       <= Tens;
      end if;
    end if;
  end process;
 
  -- Digit data selection according to active display anode:
  with anodesInt select
    digitData <= hundredthsReg when "1110",
                 tenthsReg     when "1101",
                 secondsReg    when "1011",
                 '0' & tensReg when others;
   


  -- (replace this comment by seven-segment encoding sentences)
  



           
  -- Decimal point:             
  Segments(7) <= '0' when anodesInt(2) = '0' else '1';

end Behavioral;

