----------------------------------------------------------------------------------
-- Company: UAM 
-- Design Name: stopwatch_fsm
-- Project Name: stopwatch
-- Description: 
--   State machine controlling the stopwatch's operation. The stopwatch
--   can be idle (00:00), running, showing the frozen time for a lap or 
--   stopped.
--   The block has two outputs:
--     - EnableCountOut: activated ('1') while running or showing lap time,
--       but only if EnableCountIn is active (this input signal can be
--       used as an enable for the time base counter).
--     - UpdateDisplay: when '1' the time should be shown. It is '0'
--       to freeze the display while showing a lap's time.
--   The input button signals are assumed to be already debounced. In this
--   block the transition from '0' to '1' of these signals is used to
--   change state.
-- Dependencies: 
--   (none)
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-------------------------------------------------------------------------------
-- Entity

entity stopwatch_fsm is
  port (
        Clk            : in  std_logic;  -- Clock (rising edge)
        Reset          : in  std_logic;  -- Async reset, active high
        StartStop      : in  std_logic;  -- Start/Stop button 
        Lap            : in  std_logic;  -- Lap button
        SetToZero      : in  std_logic;  -- Reset count button
        EnableCountIn  : in  std_logic;  -- Time base enable pulse train
        EnableCountOut : out std_logic;  -- EnableCountIn/'0', as f(state)
        ClearCount     : out std_logic;  -- When '1' clear all counts
        UpdateDisplay  : out std_logic   -- When '0' freeze displayed count
       );
end stopwatch_fsm;

-------------------------------------------------------------------------------
-- Architecture

architecture Behavioral of stopwatch_fsm is

  -- Type for the different states:



 
  -- Current state and next state



  
  -- Registered versions of button signals and rising edge detection signals:
  signal startStopReg  : std_logic;
  signal lapReg        : std_logic;
  signal setToZeroReg  : std_logic;
  signal startStopRise : std_logic;
  signal lapRise       : std_logic;
  signal setToZeroRise : std_logic;
  
  -- Internal fsm output: state implies stopwatch's time must run:
  signal enableCounting : std_logic;

begin

  -- Detect when each control signal gets on:
  process (Clk, Reset)
  begin
    if Reset = '1' then
      startStopReg <= '0';
      lapReg       <= '0';
      setToZeroReg <= '0';
    elsif rising_edge (Clk) then
      startStopReg <= StartStop;
      lapReg       <= Lap;
      setToZeroReg <= SetToZero;
    end if;
  end process;

  startStopRise <= StartStop and not startStopReg;
  lapRise       <= Lap       and not lapReg;
  setToZeroRise <= SetToZero and not setToZeroReg;

  -- FSM, combinational section (next state):
  
 
 
 
 
  -- FSM, register for state:
  





  -- Generate outputs:
  
  enableCounting <= '1' when ((state = RUNNING) or (state = SHOWLAP)) else '0';
  UpdateDisplay  <= '0' when (state = SHOWLAP) else '1';
  
  EnableCountOut <= EnableCountIn and enableCounting;

  ClearCount <= setToZeroRise;

end Behavioral;

